#include <Servo.h>

// Servo object
Servo jawServo;  

// Constants
const int pirPin = 8;           // Pin number for PIR sesnor
const int servoPin = 9;         // Pin number for jaw servo
const int ledLeftPin = 10;      // Pin number for LED in the left eye
const int ledRightPin = 11;     // Pin number for LED in the right eye

const int onLightTime = 5000;   // Time in milliseconds between the LEDs being activated and then being switched off
const int jawClosed = 150;      // Servo position for when the jaw is closed
const int jawOpen = 0;          // Servo position for when the jaw is open

//Global Variables
int pos = 0;                        // Current position of the jaw servo       
int pirState = LOW;                 // Current state of the PIR sensor, to handle triggering
unsigned long lastTriggerTime = 0;  // Last time that PIR sensor was triggered for eye illumination animations

// Initialise the servo, and set the pin modes for the eye LEDS and PIR sensor
void setup() {
  jawServo.attach(servoPin);
  pinMode(ledLeftPin, OUTPUT);
  pinMode(ledRightPin, OUTPUT);
  pinMode(pirPin, INPUT);
}

// The main loop of the program
// Check if the PIR sensor has been triggered, and if so, select a random "animation" to play after a short delay
void loop() {  
  if (isPirTriggered()) {
    delay(random(500, 5000));
    int animationType = random(1, 6);
    switch (animationType) {
      case 1:
        laughMouth(random(2, 10));
        break;
      case 2:
        slowOpen(random(2000, 10000)); 
        break;
      case 3:
        randomJaw(random(2, 10));
        break;
      case 4:
        eyesOnly();
        break;
      default:
        break;  
    }
  } 
  // If the eyes are on, and more time has passed than "onLightTime" since the last trigger, then turn the eyes off
  if (millis() - lastTriggerTime > onLightTime){
    eyesOff();
  }
}

// Logic for detecting if the PIR sensor has been triggered
bool isPirTriggered() {
  int val = digitalRead(pirPin);  // Read raw sensor state
  if (val == HIGH) {              // The PIR sensor detects movment
    if (pirState == LOW) {        // The previous state was LOW, so this is new trigger
      pirState = HIGH;
      return true;
    }
  } else {          
    if (pirState == HIGH){        // The previous state was HIGH, so there is no PIR movement detected, so return the state to LOW
      pirState = LOW;
    }
  }
  return false;
}

// Helper function to turn the eyes on
void eyesOn(){
  digitalWrite(ledLeftPin, HIGH);
  digitalWrite(ledRightPin, HIGH);
}

// Helper function to turn the eyes off
void eyesOff(){
  digitalWrite(ledLeftPin, LOW);
  digitalWrite(ledRightPin, LOW);
}

// ANIMATION 1 - quick opening and closing of the mouth with the eyes illuminated whilst it is opening
void laughMouth(int laughLength){
  for(int i = 0; i <= laughLength; i++) {
    eyesOn();
    jawServo.write(jawOpen);
    delay(250);
    eyesOff();
    jawServo.write(jawClosed);
    delay(250); 
  }
}

// ANIMATION 2 - very slow opening of the jaw, followed by illumination of the eyes, jaw shuts and eyes turn off after "waitTime" milliseconds
void slowOpen(int waitTime){
  for (pos = jawClosed; pos >= jawOpen; pos--) { 
    jawServo.write(pos);              
    delay(20);                       
  }
  delay(waitTime);
  eyesOn();
  jawServo.write(jawClosed);
  eyesOff();
}

// ANIMATION 3 - Full Random
void randomJaw (int iterations){
  for (int i = 0; i < iterations; i++) {
    int randomJawAngleDelta = random(2, 11);
    int randomDelay = random(5, 21);
    int randomFinalPosition = random(0, 110);
    for (pos = jawClosed; pos >= randomFinalPosition; pos -= randomJawAngleDelta) {
      jawServo.write(pos);
      if (random(1,3) == 1){
        eyesOn();
      } else {
        eyesOff();
      }
      delay(randomDelay);
    }
    for (pos = randomFinalPosition; pos <= jawClosed; pos +=randomJawAngleDelta) {
      jawServo.write(pos);
      if (random(1,3) == 1){
        eyesOn();
      } else {
        eyesOff();
      }
      delay(randomDelay);
    }
    jawServo.write(jawClosed);
    eyesOff();
  }
 
}

// ANIMATION 4 - only illuminates the eyes for the constant "onLightTime" amount of milliseconds
void eyesOnly(){
  eyesOn();
  lastTriggerTime = millis();
}
